// SEASHELL_READONLY

// This module provides an int Sequence ADT

struct sequence;

// sequence_create() returns a pointer to a new (empty) sequence.
// effects: allocates memory (caller must call sequence_destroy)
// time:    O(1)
struct sequence *sequence_create(void);

// sequence_destroy(seq) frees all memory used by the sequence seq.
// effects: seq is invalidated
// time:    O(n)
void sequence_destroy(struct sequence *seq);

// sequence_length(seq) returns the number of items currently stored in the
//   sequence seq.
// time: O(1)
int sequence_length(const struct sequence *seq);

// sequence_item_at(seq, pos) returns the item in the sequence seq at position
//   pos.
// requires: 0 <= pos < sequence_length(seq)
// time:     O(1)
int sequence_item_at(const struct sequence *seq, int pos);

// sequence_insert_at(seq, pos, itm) inserts the item itm at position pos into
//   the sequence seq. Items at position larger or equal than pos change their
//   position within seq by 1.
// requires: 0 <= pos <= sequence_length(seq)
// effects:  mutates seq
// time:     O(n) [amortized]
void sequence_insert_at(struct sequence *seq, int pos, int itm);

// sequence_remove_at(seq, pos) removes and returns the item at position pos
//   from the sequence seq. Items at position larger than pos change their
//   position within seq by 1.
// requires: 0 <= pos < sequence_length(seq)
// effects:  mutates seq
// time:     O(n)
int sequence_remove_at(struct sequence *seq, int pos);

// sequence_print(seq) prints out the items in the sequence seq.
// effects: writes to console
// time:    O(n)
void sequence_print(const struct sequence *seq);
