// SEASHELL_READONLY

#include <stdbool.h>

// This module provides an int Stack ADT

struct stack;

// stack_create() returns a pointer to a new (empty) stack.
// effects: allocates memory (caller must call stack_destroy)
// time:    O(1)
struct stack *stack_create(void);

// stack_destroy(stck) frees all memory used by the stack stck.
// effects: stck and its stored values are invalidated
// time:    O(n)
void stack_destroy(struct stack *stck);

// stack_push(stck, itm) adds the int itm onto the top of the stack stck.
// effects: mutates stck
// time:    O(n)
void stack_push(struct stack *stck, int itm);

// stack_pop(stck) removes and returns the int from the top of the stack stck.
// requires: stck must not be empty
// effects:  mutates stck
// time:     O(n)
int stack_pop(struct stack *stck);

// stack_top(stck) returns the int at the top of the stack stck.
// requires: stck must not be empty
// time:     O(1)
int stack_top(const struct stack *stck);

// stack_empty(stck) returns true if the stack stck is empty, and false 
//   otherwise.
// time: O(1)
bool stack_empty(const struct stack *stck);
