// SEASHELL_READONLY

#include <assert.h>
#include <limits.h>
#include <stdio.h>
#include "queue.h"
#include "cs136-trace.h"

// Accepted commands for the I/O-driver:
//   ENC <int:VAL> Enqueues VAL onto the end the queue.
//   DEC Removes and prints the value from the front of the queue.
//   FRONT Prints the value from the front of the queue.
//   EMPTY Prints a message indicating if the queue is empty or not.
int main(void) {
  const int cmd_enq = lookup_symbol("ENQ");
  const int cmd_deq = lookup_symbol("DEQ");
  const int cmd_front = lookup_symbol("FRONT");
  const int cmd_empty = lookup_symbol("EMPTY");
  
  struct queue *q = queue_create();
  
  int cmd = INVALID_SYMBOL;
  while ((cmd = read_symbol()) != INVALID_SYMBOL) {
    if (cmd == cmd_enq) {
      int val = INT_MIN;
      assert(scanf("%d", &val) == 1);
      queue_enqueue(q, val);
    } else if (cmd == cmd_deq) {
      int val = queue_dequeue(q);
      printf("%d\n", val);
    } else if (cmd == cmd_front) {
      int val = queue_front(q);
      printf("%d\n", val);
    } else if (cmd == cmd_empty) {
      bool empty = queue_empty(q);
      printf("Queue is%sempty\n", (empty) ? " " : " not ");
    } else {
      assert(false);
    }
  }
  
  queue_destroy(q);
}