// SEASHELL_READONLY

#include <stdbool.h>

// This module provides an int Queue ADT

struct queue;

// queue_create() returns a pointer to a new (empty) queue.
// effects: allocates memory (caller must call queue_destroy)
// time:    O(1)
struct queue *queue_create();

// queue_destroy(q) frees all memory used by the queue q.
// effects: q is invalidated
// time:    O(n)
void queue_destroy(struct queue *q);

// queue_enqueue(q, itm) adds the int itm to the end of the queue q.
// effects: mutates q
// time:    O(n)
void queue_enqueue(struct queue *q, int itm);

// queue_dequeue(q) removes and returns the int from the front of the queue q.
// requires: q must not be empty
// effects:  mutates q
// time:     O(n)
int queue_dequeue(struct queue *q);

// queue_front(q) returns the int at the front of the queue q.
// requires: q must not be empty
// time:     O(n)
int queue_front(const struct queue *q);

// queue_empty(q) returns true if the queue q is empty, and false  otherwise.
// time: O(1)
bool queue_empty(const struct queue *q);
