// SEASHELL_READONLY

#include <assert.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cs136-trace.h"
#include "sequence.h"

// === heap string <-> stack functions ========================================

void str_dest(void *str) {
  assert(str);
  free(str);
}

void str_print(const void *str) {
  assert(str);
  const char *p = str;
  printf("%s", p);
}

bool str_comp(void *str_1, void *str_2) {
  assert(str_1 && str_2);
  char *p = str_1;
  char *q = str_2;
  return strcmp(p, q) > 0;
}

// === STRING CLIENT ==========================================================

int client_string(void) {
  const int cmd_len = lookup_symbol("LENGTH");
  const int cmd_at = lookup_symbol("AT");
  const int cmd_ins = lookup_symbol("INS");
  const int cmd_rem = lookup_symbol("REM");
  const int cmd_prnt = lookup_symbol("PRINT");
  const int cmd_sort = lookup_symbol("SORT");

  struct sequence *seq = sequence_create(str_dest, str_print, str_comp);

  int cmd = INVALID_SYMBOL;
  while ((cmd = read_symbol()) != INVALID_SYMBOL) {
    if (cmd == cmd_len) {
      printf("Current sequence length is %d.\n", sequence_length(seq));
    } else if (cmd == cmd_at) {
      int idx = -1;
      assert(scanf("%d", &idx) == 1);
      str_print(sequence_item_at(seq, idx));
      printf("\n");
    } else if (cmd == cmd_ins) {
      int idx = -1;
      char tmp[100] = {'\0'};
      assert(scanf("%d%s", &idx, tmp) == 2);
      char *name = malloc((strlen(tmp) + 1) * sizeof(char));
      strcpy(name, tmp);
      sequence_insert_at(seq, idx, name);
    } else if (cmd == cmd_rem) {
      int idx = -1;
      assert(scanf("%d", &idx) == 1);
      void *data = sequence_remove_at(seq, idx);
      str_print(data);
      printf("\n");
      str_dest(data);
    } else if (cmd == cmd_prnt) {
      sequence_print(seq);
    } else if (cmd == cmd_sort) {
      sequence_sort(seq);
    } else {
      assert(false);
    }
  }

  sequence_destroy(seq);
  return 0;
}
