#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "posn.h"
#include "stack.h"

// === POSN <-> GENERIC ADT FUNCTIONS =======================================

// posn_adt_destroy(ptr) connects a generic ADT with the destroy-function
//   for a struct posn.
// effects: invalidates ptr
// time:    O(1)
void posn_adt_destroy(void *ptr) {
  assert(ptr);
  struct posn *psn = ptr;
  posn_destroy(psn);
}

// === MAIN FUNCTION ==========================================================

int main(void) {
 
  struct stack *stck = stack_create(posn_adt_destroy);
  
  printf("Stack is%sempty\n", (stack_empty(stck)) ? " " : " not ");
  
  stack_push(stck, posn_create(0, 0));
  stack_push(stck, posn_create(1, -1));
  stack_push(stck, posn_create(2, -2));
  
  printf("Stack is%sempty\n", (stack_empty(stck)) ? " " : " not ");

  const struct posn *cposn = stack_top(stck);
  posn_print(cposn);
  
  struct posn *posn = stack_pop(stck);
  posn_print(posn);
  posn_destroy(posn);
  
  posn = stack_pop(stck);
  posn_print(posn);
  posn_destroy(posn);
  
  posn = stack_pop(stck);
  posn_print(posn);
  posn_destroy(posn);

  printf("Stack is%sempty\n", (stack_empty(stck)) ? " " : " not ");
  
  stack_push(stck, posn_create(-136, 136));
  cposn = stack_top(stck);
  posn_print(cposn);

  stack_destroy(stck);
}
