#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "stack.h"

// === HEAP STRING - Helper functions =========================================

// str_create(str) creates and returns a copy of str stored in heap memory.
// effects: allocated memory (client must call str_destroy).
// time:    O(n)
static char *str_create(const char *str) {
  assert(str);
  char *val = malloc(sizeof(char) * (strlen(str) + 1));
  assert(val);
  strcpy(val, str);
  return val;
}

// str_destroy(str) releases all resources used by str.
// effects: invalidates str
// time:    O(1)
void str_destroy(char *str) {
  assert(str);
  free(str);
}

// === HEAP STRING <-> GENERIC ADT FUNCTIONS ==================================

// str_adt_destroy(ptr) releases all resources used by ptr.
// effects: invalidates ptr
// time:    O(1)
void str_adt_destroy(void *ptr) {
  assert(ptr);
  char *str = ptr;
  str_destroy(str);
}

// === MAIN FUNCTION ==========================================================

int main(void) {
 
  struct stack *stck = stack_create(str_adt_destroy);
  
  printf("Stack is%sempty\n", (stack_empty(stck)) ? " " : " not ");
  
  stack_push(stck, str_create("This"));
  stack_push(stck, str_create("Is"));
  stack_push(stck, str_create("SPARTA"));
  
  printf("Stack is%sempty\n", (stack_empty(stck)) ? " " : " not ");

  const char *cstr = stack_top(stck);
  printf("%s\n", cstr);
  
  char *str = stack_pop(stck);
  printf("%s\n", str);
  str_destroy(str);
  
  str = stack_pop(stck);
  printf("%s\n", str);
  str_destroy(str);
  
  str = stack_pop(stck);
  printf("%s\n", str);
  str_destroy(str);

  printf("Stack is%sempty\n", (stack_empty(stck)) ? " " : " not ");
  
  stack_push(stck, str_create("!!!"));

  cstr = stack_top(stck);
  printf("%s\n", cstr);

  stack_destroy(stck);
}
