#include "stack.h"
#include <assert.h>
#include <stdlib.h>

struct stack {
  int len_cur;               // current length of data; data[len_cur] also
                             //   indexes the first available array slot
  int len_max;               // maximum length of data
  void **data;               // the data, i.e., an array of pointers to the 
                             //   elements currently stored in stack
  void (*data_dest)(void *); // pointer to the destroy-function for each element
                             //   in data
};

// See stack.h for documentation.
struct stack *stack_create(void (*data_dest)(void *)) {
  struct stack *stck = malloc(sizeof(struct stack));
  stck->len_cur = 0;
  stck->len_max = 1;
  stck->data = malloc(sizeof(void *) * stck->len_max);
  stck->data_dest = data_dest;
  return stck;
}

// See stack.h for documentation.
void stack_destroy(struct stack *stck) {
  assert(stck);
  // using array index notation:
  // for (int i = 0; i < stck->len_cur; ++i) {
  //   stck->data_dest(stck->data[i]);
  // }
  // using pointer notation:
  for (void **ptr = stck->data; ptr < stck->data + stck->len_cur; ++ptr) {
    stck->data_dest(*ptr);
  }
  free(stck->data);
  free(stck);
}

// See stack.h for documentation.
void stack_push(struct stack *stck, void *val) {
  assert(stck);
  // grow (double) data if no memory available
  if (stck->len_cur == stck->len_max) {
    stck->len_max *= 2;
    stck->data = realloc(stck->data, stck->len_max * sizeof(void *));
  }
  *(stck->data + stck->len_cur) = val; // stck->data[stck->len_cur] = val;
  ++(stck->len_cur);
}

// See stack.h for documentation.
void *stack_pop(struct stack *stck) {
  assert(stck);
  --(stck->len_cur);
  if (4 * stck->len_cur < stck->len_max) {
    stck->len_max /= 4;
    stck->data = realloc(stck->data, stck->len_max * sizeof(void *));
  }
  return *(stck->data + stck->len_cur); // stck->data[stck->len_cur];
}

// See stack.h for documentation.
const void *stack_top(const struct stack *stck) {
  assert(stck);
  return *(stck->data + stck->len_cur - 1); // stck->data[stck->len_cur - 1];
}

// See stack.h for documentation.
bool stack_empty(const struct stack *stck) {
  assert(stck);
  return stck->len_cur == 0;
}
