#include <stdbool.h>

// This module provides an void * Stack ADT

struct stack;

// stack_create(data_dest) returns a pointer to a new (empty) stack. The
//   function data_dest is used to destroy the items that remain in the stack.
// effects: allocates memory (caller must call stack_destroy)
// time:    O(1)
struct stack *stack_create(void (*data_dest)(void *));

// stack_destroy(stck) frees all memory used by the stack stck and the values
//   stored within stck.
// effects: stck and its stored values are invalidated
// time:    O(n)
void stack_destroy(struct stack *stck);

// stack_push(stck, itm) adds the item itm onto the top of the stack stck.
// effects: mutates stck
// time:    O(1) [amortized]
void stack_push(struct stack *stck, void *itm);

// stack_pop(stck) removes and returns the item from the top of the stack stck.
// requires: stck must not be empty
// effects:  mutates stck
// time:     O(1) [amortized]
void *stack_pop(struct stack *stck);

// stack_top(stck) returns the item at the top of the stack stck.
// requires: stck must not be empty
// time:     O(1)
const void *stack_top(const struct stack *stck);

// stack_empty(stck) returns true if the stack stck is empty, and false 
//   otherwise.
// time: O(1)
bool stack_empty(const struct stack *stck);
