// SEASHELL_READONLY

// patient represents a patient
struct patient;

// patient_create(nm, prio) returns a new patient with name nm and priority
//   prio.
// requires: prio must be between 1 and 9 (both inclusive)
// effects:  allocates memory; client must call patient_destroy
// time:     O(n), where n is the length of nm
struct patient *patient_create(const char *nm, int prio);

// patient_destroy(pat) releases all resources used by pat.
// effects: invalidates pat
// time:    O(1)
void patient_destroy(struct patient *pat);

// patient_priority(pat) returns the priority of patient pat.
// time: O(1)
int patient_priority(const struct patient *pat);

// patient_name(pat) returns the name of patient pat.
// time: O(1)
const char *patient_name(const struct patient *pat);

// patient_print(pat) writes the information stored in pat to the console.
// effects: writes to console
// time:    O(n), where n is the length of the name of patient pat
void patient_print(const struct patient *pat);
