// SEASHELL_READONLY

#include <stdbool.h>

struct stack;

// stack_create() creates and returns a new empty stack.
// effects: allocates memory (client must call stack_destroy)
// time:    O(1)
struct stack *stack_create(void);

// stack_destroy(stck) frees all memory used by stck.
// effects: stck is no longer valid
// time:    O(1)
void stack_destroy(struct stack *stck);

// stack_is_empty(stck) returns true if stck is empty, and false otherwise.
// time: O(1)
bool stack_is_empty(const struct stack *stck);

// stack_top(stck) returns the top item in stck.
// requires: stck is not empty
// time:     O(1)
int stack_top(const struct stack *stck);

// stack_pop(stck) returns and removes the top item in stck.
// requires: stck is not empty
// time:     O(1)
int stack_pop(struct stack *stck);

// stack_push(stck, itm) adds itm to the top of stck.
// time: O(1)
void stack_push(struct stack *stck, int itm);
